//////////////////////////////////////////////////////////////////////
// System Helpers

//////////////////////////////////////////////////////////////////////
// no system routines to find a thread/semaphore/event by name (AFAIK)
//   we use the dump APIs to do a linear search ourselves

// extern int sceKernelGetThreadmanIdList(int, SceUID*, int, int*);

#define MAX_UIDS    200     /* overkill max number expected */

static SceUID FindEventFlag(const char* szFind)
{
	//int k1 = pspSdkSetK1(0);

	SceUID uids[MAX_UIDS];
	int count = 0;
	int i;

	sceKernelGetThreadmanIdList(3/*event*/, uids, sizeof(uids), &count);


	for (i = 0; i < count; i++)
	{
		SceUID uid = uids[i];
		SceKernelEventFlagInfo info;

		info.size = sizeof(info);

		int err = sceKernelReferEventFlagStatus(uid, &info);

		if (err == 0 && strcmp(info.name, szFind) == 0)
		{
			//pspSdkSetK1(k1);
			return uid;
		}
	}

	//pspSdkSetK1(k1);

	return 0;
}

static SceUID FindSemaphore(const char* szFind)
{
	//int k1 = pspSdkSetK1(0);

	SceUID uids[MAX_UIDS];
	int count = 0;
	int i;

	sceKernelGetThreadmanIdList(2/*sema*/, uids, sizeof(uids), &count);

	for (i = 0; i < count; i++)
	{
		SceUID uid = uids[i];
		SceKernelSemaInfo info;

		info.size = sizeof(info);

		int err = sceKernelReferSemaStatus(uid, &info);

		if (err == 0 && strcmp(info.name, szFind) == 0)
		{
			//pspSdkSetK1(k1);
			return uid;
		}
	}
    
	//pspSdkSetK1(k1);

	return 0;
}

//////////////////////////////////////////////////////////////////////
// dynamic binding to system entries
// you can use stubs to system entries in some cases
//  we use a direct lookup approach instead

static bool g_bFindProcError = false;

static void* FindProc(const char* szMod, const char* szLib, u32 nid)
{
	//int k1 = pspSdkSetK1(0);
	
	SceModule* modP = sceKernelFindModuleByName(szMod);

	if (modP == NULL)
	{
		//printf("Failed to find mod '%s'\n", szMod);
		g_bFindProcError = true;
		//return 0;
		//pspSdkSetK1(k1);
		return NULL;
	}

	SceLibraryEntryTable* entP = (SceLibraryEntryTable*)modP->ent_top;

	while ((u32)entP < ((u32)modP->ent_top + modP->ent_size))
	{
		if (entP->libname != NULL && strcmp(entP->libname, szLib) == 0)
		{
			// found lib
			int i;
			int count = entP->stubcount; // NOT: + entP->vstubcount;
			u32* nidtable = (u32*)entP->entrytable;

			for (i = 0; i < count; i++)
			{
				if (nidtable[i] == nid)
				{
					u32 procAddr = nidtable[count+i];
					// printf("entry found: '%s' '%s' = $%x\n", szMod, szLib, (int)procAddr);
					//pspSdkSetK1(k1);
					return (void*)procAddr;
				}
			}

			//printf("Found mod '%s' and lib '%s' but not nid=$%x\n", szMod, szLib, (unsigned int)nid);
			g_bFindProcError = true;
			//return 0;
			//pspSdkSetK1(k1);
			return NULL;
		}

		entP++;
	}

	//printf("Found mod '%s' but not lib '%s'\n", szMod, szLib);
	g_bFindProcError = true;
	
	//pspSdkSetK1(k1);

	return 0;
}


//////////////////////////////////////////////////////////////////////
